// Imperative and Object-Oriented Programming Methodology
// Uppsala University - Copyright © 2020 Henrik Alfken(Schulze), Tobias Wrigstad
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "treemap.h"

typedef struct node node_t;

struct treemap
{
  node_t* root;
};

struct node
{
  int key;
  void* value;
  node_t* left;
  node_t* right;
};

static node_t* node_create(int key, void* value)
{
  node_t* n = calloc(1, sizeof(node_t));
  n->key = key;
  n->value = value;
  return n;
}

//// Helper function for printing the keys of the tree as an array.
//// Preorder traversal.
//static void print_tree(treemap_t* tree)
//{
  //int* keys = treemap_keys(tree);
  //char* strng = calloc(512,sizeof(char));
  //int keys_size = treemap_size(tree);
  //int index = sprintf(strng, "%s", "[");
  //for (int i=0; i<keys_size-1; i++)
    //{
      //index += sprintf(&strng[index], "%d,", keys[i]);
    //}
  //if (keys_size>0) {
    //index += sprintf(&strng[index], "%d]", keys[keys_size-1]);
  //} else {
    //index += sprintf(&strng[index], "%s", "]");
  //}
  //printf("\n - The array printed as a string is: %s",strng);
  //free(strng);
  //free(keys);
//}

// Insert value v under key k in the node -- if there is
// already a value v' with key k, return v' and replace
// v' with v in the tree.
// Hint -- use this as a helper function to treemap_insert.
// Implement it as a recursive function that visits the subtrees if
// the (key, value) pair could not be inserted in THIS node.
static node_t* node_insert(node_t* n, int key, void** adr_of_value)
{
  if (n == NULL)
    {
      node_t* node_to_return = node_create(key, *adr_of_value);
      *adr_of_value = NULL;
      return node_to_return;
    }
  else if (key == n->key)
    {
      void* old_value = n->value; // Save (the pointer to) the old value.
      n->value = *adr_of_value;   // Insert the "new" value.
      *adr_of_value = old_value;  // Update value to return the old value.
    }
  else if (key < n->key)
    {
      n->left = node_insert(n->left, key, adr_of_value); // Into left subtree.
    }
  else if (key > n->key)
    {
      n->right = node_insert(n->right, key, adr_of_value); // Into right subtree.
    }
  return n;
}

/// Insert the value v under key k in the tree -- if there is
/// already a value v' with key k, return v' and replace v' with v
/// in the tree.
void* treemap_insert(treemap_t* t, int key, void* value)
{
  /// TODO: lägg till enligt spec
  /// Remember there are instructions in the exam text
  /// for how to implement insert 
  /// Hint: use a recursive help function (or double pointers)
//////printf("\n 'treemap_insert()' BEFORE, value == %p.\n",value);
  t->root = node_insert((t->root), key, &value);
//////printf(" 'treemap_insert()' AFTER,  value == %p.\n",value);
  //print_tree(t);
  return value;
}

/// Return the value associated with key k in the tree.
/// If key k is not in the tree, return NULL. 
void* treemap_lookup(treemap_t* t, int key)
{
  /// TODO: lägg till enligt spec
  node_t* n = t->root;
  while (n != NULL)
    {
      if (n->key == key) return n->value;
      if (n->key > key)
        {
          n = n->left;
        }
      else
        {
          n = n->right;
        }
    }
  return NULL; /// If we came here without finding k, it can't exist.
}

/// Creates and returns a pointer to a new empty treemap.
treemap_t* treemap_create()
{
  return calloc(1, sizeof(treemap_t)); 
}

static void node_destroy(node_t* n)
{
  /// TODO: frivillig hjälpfunktion
  /// Hint -- use this as a helper function to treemap_destroy
  /// Implement it as a recursive function that first visits
  /// the subtrees and then destroys n
  //printf("\n Function 'node_destroy()'. Visiting node %p.",n);
  //printf("\n Function 'node_destroy()'.  Left node = %p.",n->left);
  //printf("\n Function 'node_destroy()'. Right node = %p.",n->right);
  /// Recurse into left subtree
  if (n->left != NULL) node_destroy(n->left);
  /// Recurse into right subtree
  if (n->right != NULL) node_destroy(n->right);
  /// Increment the accumulator
  free(n);
}

/// Removes the treemap from memory, but does not call
/// free on any value pointer passed to it.
void treemap_destroy(treemap_t* t)
{
  /// TODO: lägg till enligt spec
  if (t->root != NULL) node_destroy(t->root);
  free(t);
}

/// Returns the true if ther is value associated with the key k in
/// the tree, else false.
bool treemap_has_key(treemap_t* t, int key)
  /// Hint: look at this function for inspiration
{
  node_t* n = t->root;
  while (n != NULL)
    {
      if (n->key == key) return true;
      else if (n->key > key)
        {
          n = n->left;
        }
      else
        {
          n = n->right;
        }
    }
  return false; /// If we came here without finding k, it can't exist.
}

/// ===================================================
/// From here on: given functions used by driver 
/// ===================================================

/// Help function to treemap_keys.
// You do not need to read or change this function.
static void node_keys(node_t* n, int** keys)
{
  if (n == NULL) return;
  **keys = n->key;
  ++*keys;
  node_keys(n->left, keys);
  node_keys(n->right, keys);
}

/// Returns an array of integers with all the keys in the
/// treemap in preorder, i.e., the following tree returns
/// [5,2,1,4,8,6,32].
/// 
///   treemap_t                  .
///      |                       .
///     _5_                      .
///    /   \                     .
///   2     8                    .
///  / \   / \                   .
/// 1   4 6   32 
// You do not need to read or look at this function.
int* treemap_keys(treemap_t* t)
{
  int* keys = calloc(treemap_size(t) + 1, sizeof(int));
  int* cursor = keys;
  node_keys(t->root, &cursor);
  return keys;
}

/// Helper function for treemap_size. Inorder traversal.
static void node_size(node_t* n, int* acc)
{
  if (n == NULL) return;
  /// Recurse into left subtree
  if (n->left != NULL) node_size(n->left, acc);
  /// Increment the accumulator
  *acc += 1;
  /// Recurse into right subtree
  if (n->right != NULL) node_size(n->right, acc);
}

/// Returns the number of (key, value) pairs in the tree.
/// The empty tree has size 0. The following tree has size 7.
/// 
///   treemap_t                  .
///      |                       .
///     _5_                      .
///    /   \                     .
///   2     8                    .
///  / \   / \                   .
/// 1   4 6   32 
int treemap_size(treemap_t* t)
{
  int result = 0;
  node_size(t->root, &result);
  return result;
}
