// Imperative and Object-Oriented Programming Methodology
// Uppsala University - Copyright © 2020 Henrik Alfken(Schulze), Tobias Wrigstad
package org.ioopm.planner;

import java.util.*;

/// TODO: lägg till nya bågar --- OBS!! Du måste se till att de
/// nya bågarna används när grafen byggs upp också, vilket sker i addLine().
/// Du får själv lista ut vilken typ av båge som skall användas var.
// Enklast här kan vara att kåppy-pejsta Edge-klassen ...
// NEVER EVER use default (package private) access. I do it here ONLY to have all code in one file:
class LineEdge extends Edge {
  private final int lineNr;

  public LineEdge(final Node n1, final Node n2, final int weight, final int lineNr) {
    super(n1, n2, weight);
    this.lineNr = lineNr;
  }

  @Override
  public String info() {
    return "Linje " + lineNr + ", " + super.info();
  }
}

// NEVER EVER use default (package private) access. I do it here ONLY to have all code in one file:
class StopOver extends Edge {
  private static final int WEIGHT = 8;

  public StopOver(final Node n1, final Node n2) {
    super(n1, n2, WEIGHT);
  }

  @Override
  public String info() {
    return " - Byte, " + super.info();
  }
}

public class Trip {
  private final Node start;
  private final Node destination;
  private List<Node> route;

  /// If line == [a, b, c] and weights == [8, 3], then
  /// this creates the following network:
  ///
  ///      8     3
  ///   a --- b --- c
  ///
  /// and adds a,b,c to the set of nodes in the network.
  public static void addLine(Set<Node> nodes, Node[] line, int[] weigths, int lineNumber) {
    /// TODO: Extend from Model 1 to Model 3
    /// Tip: you can ask if foo is in the set with node.contains(foo)
    /// Tip: Examples of node construction can be found in e.g. PrinsEugen.java
    for (int i = 0; i < line.length; ++i) {
      /// n is the n from the algorithm in the description
      Node n = line[i];
      /// existing is E from the algorithm in the description
      Set<Node> existing = Network.getNodesByName(line[i].name());
      /// TODO: write the rest of the logic here
      // 1.1. Låt E vara en mängd av alla _existerande_ noder i nätverket med samma namn som n.
      // 1.1.1. Om E inte är tom,
      // 1.1.1.1 skapa en ny nod n1 med samma namn som n,
      // 1.1.1.2 för varje nod e i E, koppla ihop e och n1 med en bytes-båge (= StopOver)
      //          (en ny båge för varje koppling),
      // 1.1.1.3 ersätt n med n1 i linjen (alltså arrayen av noder).
      if (existing.isEmpty() == false) {
        Node n1 = new Node(n.name());
        for (Node e : existing) {
          StopOver stopOver = new StopOver(e, n1);
          e.connectTo(stopOver);
          n1.connectTo(stopOver);
        }
        line[i] = n1;
      }
    }
    /// FWIW, the loop below shows how to connect two nodes with an Edge edge.
    // 2. Fortsätt med existerande logik som addLine() men fundera på vilken sorts båge
    // som skall användas..
    //  - Well, to be able to write out the line number, this Edge needs to be of type LineEdge.
    /// (Connect the two nodes with a new Edge edge)
    /// Connect the two nodes with a new LineEdge, called lineEdge:
    for (int i = 0; i < weigths.length; ++i) {
      Node from = line[i];
      Node to = line[i + 1];
      LineEdge lineEdge = new LineEdge(from, to, weigths[i], lineNumber);
      from.connectTo(lineEdge);
      to.connectTo(lineEdge);
      nodes.add(from);
      nodes.add(to);
    }
  }

  /// Calculates the travel time for the proposed route
  public String routeToString() {
    /// TODO: Add route information (times, lines, stop-overs)
    /// Note, you can get information from edges using the info() method
    String result = "";
    final LinkedList<Node> trip = new LinkedList<>(route);
    Node from = trip.pop();
    while (trip.isEmpty() == false) {
      Node to = trip.pop();
      result += from.name() + "\n";
      result += "   " + from.getEdgeTo(to).info() + "\n";
      from = to;
    }
    result += from.name() + "\n- - - - - - - - - - - - - - - - -";
    return result;
  }

  public int numberOfStopOvers() {
    /// TODO: Finish
    int stopOvers = 0;
    final LinkedList<Node> trip = new LinkedList<>(route);
    Node from = trip.pop();
    while (trip.isEmpty() == false) {
      Node to = trip.pop();
//      result += from.name() + "\n";
      Edge edge = from.getEdgeTo(to);
      if (edge instanceof StopOver) { ++stopOvers; }
      from = to;
    }
    return stopOvers;
  }

  /// No need to change or look at the functions below this line!
  /// Calculates the travel time for the proposed route -- should not need to change this
  public int travelTime() {
    int time = 0;
    final LinkedList<Node> tmp = new LinkedList<>(route);
    Node from = tmp.pop();
    while (tmp.isEmpty() == false) {
      Node to = tmp.pop();
      time += from.getEdgeTo(to).weight();
      from = to;
    }
    return time;
  }

  /// Create a new trip
  public Trip(final Node start, final Node destination) {
    this.start = start;
    this.destination = destination;
  }

  /// Get the intended start
  public Node start() {
    return this.start;
  }

  /// Get the intended destination
  public Node destination() {
    return this.destination;
  }

  /// Sets the proposed route for the trip
  public void setRoute(List<Node> route) {
    this.route = route;
  }
}
